@@ -123,6 +123,7 @@ struct _GrdSessionRdp
   GrdRdpSAMFile *sam_file;
   uint32_t rdp_error_info;
   GrdRdpScreenShareMode screen_share_mode;
+  gboolean session_stopped;
 
   GMutex rdp_flags_mutex;
   RdpPeerFlag rdp_flags;
@@ -2011,9 +2012,20 @@ socket_thread_func (gpointer data)
 
   while (TRUE)
     {
+      GrdRdpNetworkAutodetection *network_autodetection =
+        rdp_peer_context->network_autodetection;
+      gboolean pending_bw_measure_stop = FALSE;
+      HANDLE bw_measure_stop_event = NULL;
+
       n_events = 0;
 
       events[n_events++] = session_rdp->stop_event;
+      if (network_autodetection)
+        {
+          bw_measure_stop_event =
+            grd_rdp_network_autodetection_get_bw_measure_stop_event_handle (network_autodetection);
+          events[n_events++] = bw_measure_stop_event;
+        }
 
       n_freerdp_handles = peer->GetEventHandles (peer, &events[n_events],
                                                  32 - n_events);
@@ -2068,6 +2080,12 @@ socket_thread_func (gpointer data)
             break;
         }
 
+      if (bw_measure_stop_event)
+        {
+          pending_bw_measure_stop = WaitForSingleObject (bw_measure_stop_event,
+                                                         0) == WAIT_OBJECT_0;
+        }
+
       if (WaitForSingleObject (channel_event, 0) == WAIT_OBJECT_0 &&
           !WTSVirtualChannelManagerCheckFileDescriptor (vcm))
         {
@@ -2075,6 +2093,9 @@ socket_thread_func (gpointer data)
           handle_client_gone (session_rdp);
           break;
         }
+
+      if (pending_bw_measure_stop)
+        grd_rdp_network_autodetection_bw_measure_stop (network_autodetection);
     }
 
   if (session_rdp->hwaccel_nvidia)
@@ -2091,7 +2112,7 @@ graphics_thread_func (gpointer data)
   if (session_rdp->hwaccel_nvidia)
     grd_hwaccel_nvidia_push_cuda_context (session_rdp->hwaccel_nvidia);
 
-  while (WaitForSingleObject (session_rdp->stop_event, 0) == WAIT_TIMEOUT)
+  while (!session_rdp->session_stopped)
     g_main_context_iteration (session_rdp->graphics_context, TRUE);
 
   if (session_rdp->hwaccel_nvidia)
@@ -2182,6 +2203,8 @@ grd_session_rdp_stop (GrdSession *session)
   g_debug ("Stopping RDP session");
 
   unset_rdp_peer_flag (session_rdp, RDP_PEER_ACTIVATED);
+  session_rdp->session_stopped = TRUE;
+
   if (WaitForSingleObject (session_rdp->stop_event, 0) == WAIT_TIMEOUT)
     {
       freerdp_set_error_info (peer->context->rdp,
@@ -2202,12 +2225,8 @@ grd_session_rdp_stop (GrdSession *session)
 
   if (session_rdp->graphics_thread)
     {
-      uint32_t status;
-
       g_assert (session_rdp->graphics_context);
-
-      status = WaitForSingleObject (session_rdp->stop_event, 0);
-      g_assert (status != WAIT_TIMEOUT);
+      g_assert (session_rdp->session_stopped);
 
       g_main_context_wakeup (session_rdp->graphics_context);
       g_clear_pointer (&session_rdp->graphics_thread, g_thread_join);
